@@ -17,6 +17,7 @@
 #    along with GNOME Keysign.  If not, see <http://www.gnu.org/licenses/>.
 from __future__ import unicode_literals
 
+import base64
 import logging
 import os  # The SigningKeyring uses os.symlink for the agent
 from subprocess import check_output
@@ -24,8 +25,10 @@ import sys
 from tempfile import mkdtemp
 import platform
 
+import dbus
 import gpg
 from gpg.constants import PROTOCOL_OpenPGP
+from gpg.errors import GPGMEError
 
 
 from .gpgkey import Key, UID
@@ -35,16 +38,6 @@ texttype = unicode if sys.version_info.major < 3 else str
 log = logging.getLogger(__name__)
 
 
-major, minor, patch = map(int, gpg.version.versionlist)
-# Due to https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=884900
-# we'd crash if we're accessing signatures on a key.
-# With this test we can try to avoid that as much as possible.
-on32bit = platform.architecture()[0] == "32bit"
-crashing_gpgme = on32bit  and  major <= 1  and  minor <= 10  and  patch <= 0
-log.info("Detected gpgme (%d.%d.%d). %s", major, minor, patch, "And it might crash" if crashing_gpgme else "")
-
-
-
 #####
 ## INTERNAL API
 ##
@@ -256,7 +249,7 @@ class DirectoryContext(gpg.Context):
 
 class TempContext(DirectoryContext):
     def __init__(self):
-        self.homedir = mkdtemp()
+        self.homedir = mkdtemp(suffix="-gnome-keysign")
         super(TempContext, self).__init__(homedir=self.homedir)
 
     def __del__(self):
@@ -420,13 +413,55 @@ def minimise_key(keydata):
         minimised_key = sink.read()
         return minimised_key
 
+def local_sign_keydata(keydata, expires_in=60*60*24*1, error_cb=None, homedir=None):
+    """Produces non-exportable and expiring signatures.
+    This can be useful if we want to enable the user to send an email to 
+    the other party right away, without waiting for the protocol to have 
+    completed. By letting the signature expire, we limit the time in 
+    which a wrongly signed key is harmful. This is a challenge for thx
+    UX, because sending emails will stop working pretty much out of 
+    the blue. But it can hardly be any worse than it is now.
+    And the app ought to inform the user about the fact that it's only 
+    ephemeral.
+
+    Returns: nothing
+    """
+    ctx = DirectoryContext(homedir)
+
+    tmpctx = TempContext()
+    available_secret_keys = [key for key in ctx.keylist(secret=True)
+        if not key.disabled or key.revoked or key.invalid or key.expired]
+    log.debug('Setting available sec keys to: %r', available_secret_keys)
+    ctx.signers = available_secret_keys
+
+    tmpctx.op_import(keydata)
+    result = tmpctx.op_import_result()
+    if result.considered != 1 and result.imported != 1:
+        raise ValueError("Expected to load exactly one key. %r", result)
+    else:
+        imports = result.imports
+        assert len(imports) == 1
+        fpr = result.imports[0].fpr
+
+        ctx.op_import(keydata)
+        key = ctx.get_key(fpr)
+        # We need to sign in the regular context, because gpgme does not
+        # export local signatures from a keyring.
+        ctx.key_sign(key, local=True, expires_in=expires_in)
+        # Unfortunately, key_sign does not report back how many
+        # signatures were produced (or not produced...)
+        # It may raise an error, but I have yet to see that it does...
+        log.info("Locally signed key %s with an exiry in %d secods", fpr, expires_in)
+
+
 def sign_keydata_and_encrypt(keydata, error_cb=None, homedir=None):
     oldctx = DirectoryContext(homedir)
     ctx = TempContextWithAgent(oldctx)
     # We're trying to sign with all available secret keys
     available_secret_keys = [key for key in ctx.keylist(secret=True)
-        if not key.disabled or key.revoked or key.invalid or key.expired]
-    log.debug('Setting available sec keys to: %r', available_secret_keys)
+        if not (key.disabled or key.revoked or key.invalid or key.expired)]
+    log.debug('Setting available sec keys to (%d): %r',
+        len(available_secret_keys), available_secret_keys)
     ctx.signers = available_secret_keys
 
     ctx.op_import(minimise_key(keydata))
@@ -458,11 +493,16 @@ def sign_keydata_and_encrypt(keydata, error_cb=None, homedir=None):
             if uid.revoked or uid.invalid:
                 continue
             else:
+                log.debug("The UID %s has %d signatures",
+                    uid, len(uid.signatures))
+                if len(uid.signatures) < 2:
+                    log.error("We seem to not have produced signatures correctly. "
+                        "%s has less than 2 signatures: %s",
+                        uid, uid.signatures,
+                    )
+
                 uid_data = UIDExport(signed_keydata, i)
-                # FIXME: Check whether this bug is resolved and the remove this conditional
-                # https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=884900
-                if not crashing_gpgme:
-                    log.debug("Data for uid %d: %r, sigs: %r %r", i, uid, uid.signatures, uid_data)
+                log.debug("Data for uid %d: %r, sigs: %r %r", i, uid, uid.signatures, uid_data)
 
                 ciphertext, _, _ = ctx.encrypt(plaintext=uid_data,
                                                recipients=[key],
@@ -470,4 +510,49 @@ def sign_keydata_and_encrypt(keydata, error_cb=None, homedir=None):
                                                # in order for it to work out of the box
                                                always_trust=True,
                                                sign=False)
-                yield (UID.from_gpgme(uid), ciphertext)
+                yield (UID.from_gpgme(uid), ciphertext, uid_data)
+
+
+def import_signature(encrypted_sig, homedir=None):
+    ctx = DirectoryContext(homedir)
+
+    # Check if we are really importing a signature
+    temp_ctx = TempContextWithAgent(ctx)
+    signature = temp_ctx.decrypt(encrypted_sig)
+    temp_ctx.op_import(signature[0])
+    result = temp_ctx.op_import_result()
+
+    if result.imported != 0:
+        log.warning("Trying to import a new key instead of a signature!")
+        raise GPGMEError
+
+    if result.new_signatures == 0 or result.revocations != 0 or result.new_sub_keys != 0:
+        log.warning("The signature that we were importing is not as we expected!")
+        raise GPGMEError
+
+    signature = ctx.decrypt(encrypted_sig)
+
+    # Try Seahorse DBus
+    name = "org.gnome.seahorse"
+    path = "/org/gnome/seahorse/keys"
+    bus = dbus.SessionBus()
+    result = []
+
+    try:
+        proxy = bus.get_object(name, path)
+    except dbus.exceptions.DBusException:
+        log.debug("Seahorse DBus is not available")
+    else:
+        iface = "org.gnome.seahorse.KeyService"
+        gpg_iface = dbus.Interface(proxy, iface)
+        payload = base64.b64encode(signature[0]).decode('latin-1')
+        payload = '\n'.join(payload[i:(i + 64)] for i in range(0, len(payload), 64))
+        payload = "-----BEGIN PGP PUBLIC KEY BLOCK-----\n\n" + payload + "\n-----END PGP PUBLIC KEY BLOCK-----"
+        result = gpg_iface.ImportKeys("openpgp", payload)
+
+    # If Seahorse failed we try op_import
+    if len(result) < 1:
+        ctx.op_import(signature[0])
+        result = ctx.op_import_result()
+        if len(result.imports) < 1:
+            raise GPGMEError
