@@ -35,13 +35,8 @@ lang = gettext.translation(APP, DIR, fallback=True)
 
 
 from locale import gettext as _
-# TRANSLATORS: Please include your locale, e.g. "de". We're trying to debug native gettext
-log.debug (_("Translated for gettext (C)"))
 
 try:
     _ = lang.ugettext
 except AttributeError:
     log.info("Cannot get ugettext from lang: %r", lang, exc_info=True)
-
-# TRANSLATORS: Please include your locale, e.g. "de". We're trying to debug pure python gettext
-log.debug (_("Translated for Python (C)"))
