@@ -1,13 +1,21 @@
 #!/usr/bin/env python
 
 import logging
+import mailbox
 import os
 import signal
 
+try:
+    from urllib.parse import unquote
+except ImportError:
+    from urllib import unquote
+
 import gi
 gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 from gi.repository import GLib
+from gi.repository import Gdk
+from gpg import errors
 from wormhole.errors import ServerConnectionError, LonelyError, WrongPasswordError
 if __name__ == "__main__":
     from twisted.internet import gtk3reactor
@@ -30,6 +38,7 @@ if  __name__ == "__main__" and __package__ is None:
 from .keylistwidget import KeyListWidget
 from .KeyPresent import KeyPresentWidget
 from .offer import Offer
+from .util import get_attachments
 from . import gpgmeh
 # We import i18n to have the locale set up for Glade
 from .i18n import _
@@ -42,6 +51,9 @@ except ImportError:
     BluetoothOffer = None
 
 
+DRAG_ACTION = Gdk.DragAction.COPY
+
+
 class SendApp:
     """Common functionality needed when building the sending part
     
@@ -92,12 +104,47 @@ class SendApp:
         self.notify = None
         self.internet_option = False
 
+        # Add drag and drop to the keys list widget
+        builder.connect_signals(self)
+        self.label = builder.get_object("keys_listbox")
+        self.label.drag_dest_set(Gtk.DestDefaults.ALL, [], DRAG_ACTION)
+        self.label.drag_dest_set_target_list(None)
+        self.label.drag_dest_add_text_targets()
+        self.label.drag_dest_add_uri_targets()
+
+    def on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
+        if self.notify is not None:
+            log.debug("We are trying to send a key, no imports at this stage")
+            return
+
+        filename = data.get_text()
+        # If we don't have a filename it means that the user maybe dropped
+        # an attachment or an entire email.
+        if not filename:
+            filename = data.get_data().decode("utf-8")
+
+        filename = unquote(filename)
+        filename = filename[7:].strip('\r\n\x00')  # remove file://, \r\n and NULL
+        log.info("Received file: %s", filename)
+        signatures = get_attachments(filename)
+        if not signatures:
+            with open(filename, "rb") as si:
+                signatures.append(si.read())
+
+        try:
+            for signature in signatures:
+                gpgmeh.import_signature(signature)
+            self.signature_imported()
+        except errors.GPGMEError as e:
+            log.exception("Could not import signatures")
+            self.signature_import_error()
+
     @inlineCallbacks
     def on_key_activated(self, widget, key):
         # Deactivate any old connection attempt
         self._deactivate_timer()
         self.deactivate()
-        self.klw.ib.hide()
+        self.klw.ib_internet.hide()
         self.key = key
         log.info("Activated key %r", key)
         ####
@@ -152,17 +199,34 @@ class SendApp:
         else:
             self.show_result(success, message)
 
+    def on_ib_closed(self, widget, data):
+        if Gtk.ResponseType.CLOSE == data:
+            widget.hide()
+
     def slow_connection(self):
-        self.klw.label_ib.set_label(_("Still trying to get a connection to the Internet. "
-                                      "It appears to be slow or unavailable."))
-        self.klw.ib.show()
+        self.klw.label_ib_internet.set_label(_("Still trying to get a connection to the Internet. "
+                                             "It appears to be slow or unavailable."))
+        self.klw.ib_internet.show()
         log.info("Slow Internet connection")
 
     def no_connection(self):
-        self.klw.label_ib.set_label(_("There isn't an Internet connection!"))
-        self.klw.ib.show()
+        self.klw.label_ib_internet.set_label(_("There isn't an Internet connection!"))
+        self.klw.ib_internet.show()
         log.info("No Internet connection")
 
+    def signature_imported(self):
+        self.klw.image_ib_import.set_from_icon_name(Gtk.STOCK_OK, Gtk.IconSize.BUTTON)
+        self.klw.label_ib_import.set_label(_("The signature has been successfully imported!"))
+        self.klw.ib_import.show()
+        log.info("Signature imported")
+
+    def signature_import_error(self):
+        self.klw.image_ib_import.set_from_icon_name(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.BUTTON)
+        self.klw.label_ib_import.set_label(_("An error occurred while trying to import the signature.\n"
+                                           "Please double check the correctness of the chosen signature."))
+        self.klw.ib_import.show()
+        log.info("Signature import error")
+
     def create_keypresent(self, discovery_code, discovery_data):
         self._deactivate_timer()
         log.info("Use this for discovering the other key: %r", discovery_data)
@@ -176,7 +240,7 @@ class SendApp:
         self.stack_saved_visible_child = self.stack.get_visible_child()
         self.stack.set_visible_child(self.kpw)
         log.debug('Setting kpw: %r', self.kpw)
-        self.klw.ib.hide()
+        self.klw.ib_internet.hide()
         self.klw.code_spinner.stop()
 
     def show_result(self, success, message):
@@ -217,7 +281,7 @@ class SendApp:
     def set_internet_option(self, value):
         self._deactivate_timer()
         self.deactivate()
-        self.klw.ib.hide()
+        self.klw.ib_internet.hide()
         self.klw.code_spinner.stop()
         self.internet_option = value
 
